Type: Package
Package: scDiagnostics
Title: Cell type annotation diagnostics
Version: 1.4.0
Authors@R: c(
    person("Anthony", "Christidis", role = c("aut", "cre"), 
            email = "anthony-alexander_christidis@hms.harvard.edu", 
            comment = c(ORCID = "0000-0002-4565-6279")),
    person("Andrew", "Ghazi", role = "aut"),
    person("Smriti", "Chawla", role = "aut"),
    person("Nitesh", "Turaga", role = "ctb"),
    person("Ludwig", "Geistlinger", role = "aut"),
    person("Robert", "Gentleman", role = "aut")
  )
Description: The scDiagnostics package provides diagnostic plots to
    assess the quality of cell type assignments from single cell gene
    expression profiles. The implemented functionality allows to
    assess the reliability of cell type annotations, investigate gene
    expression patterns, and explore relationships between different
    cell types in query and reference datasets allowing users to
    detect potential misalignments between reference and query
    datasets. The package also provides visualization capabilities for
    diagnostics purposes.    
License: Artistic-2.0
URL: https://github.com/ccb-hms/scDiagnostics
BugReports: https://github.com/ccb-hms/scDiagnostics/issues
Depends: R (>= 4.4.0)
Imports: SingleCellExperiment, methods, isotree, FNN, igraph, ggplot2,
        GGally, ggridges, SummarizedExperiment, ranger, transport,
        cramer, rlang, bluster, scales, MASS, stringr, Matrix,
        grDevices
Suggests: AUCell, BiocStyle, knitr, rmarkdown, scran, scRNAseq,
        SingleR, celldex, scuttle, scater, dplyr, ComplexHeatmap, grid,
        testthat (>= 3.0.0)
VignetteBuilder: knitr
biocViews: Annotation, Classification, Clustering, GeneExpression,
        RNASeq, SingleCell, Software, Transcriptomics
Encoding: UTF-8
LazyDataCompression: xz
RoxygenNote: 7.3.2
Config/testthat/edition: 3
LazyData: true
git_url: https://git.bioconductor.org/packages/scDiagnostics
git_branch: RELEASE_3_22
git_last_commit: 7dca061
git_last_commit_date: 2025-10-29
Repository: Bioconductor 3.22
Date/Publication: 2025-10-29
NeedsCompilation: no
Packaged: 2025-10-30 06:21:07 UTC; biocbuild
Author: Anthony Christidis [aut, cre] (ORCID:
    <https://orcid.org/0000-0002-4565-6279>),
  Andrew Ghazi [aut],
  Smriti Chawla [aut],
  Nitesh Turaga [ctb],
  Ludwig Geistlinger [aut],
  Robert Gentleman [aut]
Maintainer: Anthony Christidis <anthony-alexander_christidis@hms.harvard.edu>
