# Generated by roxygen2: do not edit by hand

S3method(plot,calculateAveragePairwiseCorrelationObject)
S3method(plot,calculateCellDistancesObject)
S3method(plot,calculateCellSimilarityPCAObject)
S3method(plot,calculateDiscriminantSpaceObject)
S3method(plot,calculateGraphIntegrationObject)
S3method(plot,calculateSIRSpaceObject)
S3method(plot,calculateTopLoadingGeneShiftsObject)
S3method(plot,calculateWassersteinDistanceObject)
S3method(plot,compareMarkersObject)
S3method(plot,comparePCAObject)
S3method(plot,comparePCASubspaceObject)
S3method(plot,detectAnomalyObject)
S3method(plot,regressPCObject)
export(boxplotPCA)
export(calculateAveragePairwiseCorrelation)
export(calculateCategorizationEntropy)
export(calculateCellDistances)
export(calculateCellDistancesSimilarity)
export(calculateCellSimilarityPCA)
export(calculateCramerPValue)
export(calculateDiscriminantSpace)
export(calculateGraphIntegration)
export(calculateHVGOverlap)
export(calculateHotellingPValue)
export(calculateMMDPValue)
export(calculateSIRSpace)
export(calculateTopLoadingGeneShifts)
export(calculateVarImpOverlap)
export(calculateWassersteinDistance)
export(compareMarkers)
export(comparePCA)
export(comparePCASubspace)
export(detectAnomaly)
export(histQCvsAnnotation)
export(plotCellTypeMDS)
export(plotCellTypePCA)
export(plotGeneExpressionDimred)
export(plotGeneSetScores)
export(plotMarkerExpression)
export(plotPairwiseDistancesDensity)
export(plotQCvsAnnotation)
export(processPCA)
export(projectPCA)
export(projectSIR)
export(regressPC)
import(SingleCellExperiment)
importFrom(SummarizedExperiment,assay)
importFrom(ggplot2,aes)
importFrom(ggplot2,element_blank)
importFrom(ggplot2,element_line)
importFrom(ggplot2,element_rect)
importFrom(ggplot2,element_text)
importFrom(ggplot2,facet_grid)
importFrom(ggplot2,geom_col)
importFrom(ggplot2,geom_point)
importFrom(ggplot2,geom_text)
importFrom(ggplot2,geom_tile)
importFrom(ggplot2,ggplot)
importFrom(ggplot2,labs)
importFrom(ggplot2,margin)
importFrom(ggplot2,scale_fill_gradient2)
importFrom(ggplot2,scale_fill_manual)
importFrom(ggplot2,scale_y_discrete)
importFrom(ggplot2,theme)
importFrom(ggplot2,theme_minimal)
importFrom(ggplot2,unit)
importFrom(ggplot2,ylim)
importFrom(ggridges,geom_density_ridges)
importFrom(grDevices,dev.off)
importFrom(grDevices,pdf)
importFrom(methods,is)
importFrom(rlang,.data)
importFrom(stats,aggregate)
importFrom(stats,approxfun)
importFrom(stats,as.formula)
importFrom(stats,cancor)
importFrom(stats,cmdscale)
importFrom(stats,complete.cases)
importFrom(stats,cor)
importFrom(stats,cov)
importFrom(stats,density)
importFrom(stats,dist)
importFrom(stats,formula)
importFrom(stats,mahalanobis)
importFrom(stats,median)
importFrom(stats,model.matrix)
importFrom(stats,na.omit)
importFrom(stats,p.adjust)
importFrom(stats,predict)
importFrom(stats,pt)
importFrom(stats,qchisq)
importFrom(stats,qnorm)
importFrom(stats,quantile)
importFrom(stats,reorder)
importFrom(stats,reshape)
importFrom(stats,sd)
importFrom(stats,setNames)
importFrom(stats,var)
importFrom(stats,wilcox.test)
importFrom(tools,toTitleCase)
importFrom(utils,combn)
importFrom(utils,head)
importFrom(utils,tail)
